"use strict";
// Copyright 2018 Knowledge Expert SA
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.Browser = void 0;
const selenium_webdriver_1 = require("selenium-webdriver");
const collection_1 = require("./collection");
const configuration_1 = require("./configuration");
const element_1 = require("./element");
const entity_1 = require("./entity");
const BrowserWebElementByJs_1 = require("./locators/BrowserWebElementByJs");
const BrowserWebElementByLocator_1 = require("./locators/BrowserWebElementByLocator");
const BrowserWebElementsByJs_1 = require("./locators/BrowserWebElementsByJs");
const BrowserWebElementsByLocator_1 = require("./locators/BrowserWebElementsByLocator");
const queries_1 = require("./queries");
const extensions_1 = require("./utils/extensions");
var isAbsoluteUrl = extensions_1.Extensions.isAbsoluteUrl;
class Browser extends entity_1.Entity {
    static configuredWith() {
        return configuration_1.Customized.browser();
    }
    static chromeWith() {
        return Browser
            .configuredWith().driver(new selenium_webdriver_1.Builder().withCapabilities(selenium_webdriver_1.Capabilities.chrome()).build());
    }
    static chrome() {
        return Browser.chromeWith().build();
    }
    constructor(configuration = {}) {
        super(new configuration_1.Configuration(configuration));
    }
    with(customConfig) {
        return new Browser(new configuration_1.Configuration(Object.assign(Object.assign({}, this.configuration), customConfig)));
    }
    get driver() {
        return typeof this.configuration.driver === 'function'
            ? this.configuration.driver()
            : this.configuration.driver;
    }
    // eslint-disable-next-line class-methods-use-this
    toString() {
        return 'browser';
    }
    /* Elements */
    element(located, customized) {
        const configuration = customized === undefined
            ? this.configuration
            : new configuration_1.Configuration(Object.assign(Object.assign({}, this.configuration), customized));
        if (located instanceof selenium_webdriver_1.By || typeof located === 'string') {
            const by = located instanceof selenium_webdriver_1.By ? located : this.configuration._locationStrategy(located);
            const locator = new BrowserWebElementByLocator_1.BrowserWebElementByLocator(by, this);
            return new element_1.Element(locator, configuration);
        }
        const locator = new BrowserWebElementByJs_1.BrowserWebElementByJs(this, located.script, located.args);
        return new element_1.Element(locator, configuration);
    }
    all(located, customized) {
        const configuration = customized === undefined
            ? this.configuration
            : new configuration_1.Configuration(Object.assign(Object.assign({}, this.configuration), customized));
        if (located instanceof selenium_webdriver_1.By || typeof located === 'string') {
            const by = located instanceof selenium_webdriver_1.By ? located : this.configuration._locationStrategy(located);
            const locator = new BrowserWebElementsByLocator_1.BrowserWebElementsByLocator(by, this);
            return new collection_1.Collection(locator, configuration);
        }
        const locator = new BrowserWebElementsByJs_1.BrowserWebElementsByJs(this, located.script, located.args);
        return new collection_1.Collection(locator, configuration);
    }
    /* Commands */
    async executeScript(script, ...args) {
        const wrappedScript = `var args = arguments;${script instanceof Function
            ? `return (${script.toString()})(document, args, window);`
            : `return (function(document, args, window) { ${script} })(document, args, window);`}`;
        return this.driver.executeScript(wrappedScript, ...args);
    }
    async open(relativeOrAbsoluteUrl) {
        if (this.configuration.windowHeight && this.configuration.windowWidth) {
            await this.resizeWindow(parseInt(this.configuration.windowWidth, 10), parseInt(this.configuration.windowHeight, 10));
        }
        const absoluteUrl = isAbsoluteUrl(relativeOrAbsoluteUrl)
            ? relativeOrAbsoluteUrl
            : this.configuration.baseUrl + relativeOrAbsoluteUrl;
        await this.driver.get(absoluteUrl);
        return this;
    }
    async resizeWindow(width, height) {
        await this.driver.manage().window().setRect({ width, height });
        return this;
    }
    async screenshot() {
        return this.configuration.fullPageScreenshot
            ? Buffer.from(await this.driver.takeScreenshot(), 'base64') // todo: change to fullPageScreenshot(driver);
            : Buffer.from(await this.driver.takeScreenshot(), 'base64');
    }
    async closeCurrentTab() {
        await this.driver.close();
        return this;
    }
    async quit() {
        await this.driver.quit();
    }
    async refresh() {
        await this.driver.navigate().refresh();
    }
    async back() {
        await this.driver.navigate().back();
    }
    async forward() {
        await this.driver.navigate().forward();
    }
    async goToNextTab() {
        await this.driver
            .switchTo()
            .window(await queries_1.query.nextTab(this));
        return this;
    }
    async goToPreviousTab() {
        await this.driver
            .switchTo()
            .window(await queries_1.query.previousTab(this));
        return this;
    }
    async goToTab(indexOrId) {
        if (typeof indexOrId === 'number') {
            await this.driver
                .switchTo()
                .window(await (queries_1.query.tab(indexOrId)(this)));
        }
        else {
            await this.driver
                .switchTo()
                .window(indexOrId);
        }
        return this;
    }
    /*    async switchToFrame(frameElement: Element): Promise<Browser> {
            await this.wait.command(async browser => {
                browser.driver.switchTo().frame(await frameElement.getWebElement());
            });
            return this;
        } */
    async switchToDefaultFrame() {
        await this.driver.switchTo().defaultContent();
        return this;
    }
    // todo: cache is not the same as LocalAndSessionStorage; so we have to be verbose in name; but do we need it then?
    /*    async clearLocalAndSessionStorageAndCookies(): Promise<Browser> {
            await this.driver.executeScript('window.localStorage.clear();')
                .catch(ignored => {});
            await this.driver.executeScript('window.sessionStorage.clear();')
                .catch(ignored => {});
            await this.driver.manage().deleteAllCookies()
                .catch(ignored => {});
            return this;
        } */
    async clearLocalStorage() {
        await this.driver.executeScript('window.localStorage.clear();')
            .catch(ignored => { });
        return this;
    }
    async clearSessionStorage() {
        await this.driver.executeScript('window.sessionStorage.clear();')
            .catch(ignored => { });
        return this;
    }
    async clearCookies() {
        await this.driver.manage().deleteAllCookies()
            .catch(ignored => { });
        return this;
    }
    /*
        async deleteCookie(name: string): Promise<Browser> {
            await this.driver.manage().deleteCookie(name);
            return this;
        } */
    get alert() {
        return this.driver.switchTo().alert();
    }
}
exports.Browser = Browser;
//# sourceMappingURL=browser.js.map